Option Explicit
' Module: File Operations
' Purpose:
'     Handles all interactions with the file system as
'     well as keeps track of what file is currently open
'     and the original contents of that file.

Dim FILE_CONTENTS As String
Dim PREVIOUS_PATH As String
Dim PREVIOUS_FILE_NAME As Variant
Dim PREVIOUS_FULL_PATH As String

Function ConvertTabsToSpaces$ (DataToConvert$)
    
    'hour glass mouse cursor during processing
    Screen.MousePointer = 11

    'Replace any tab characters with 4 spaces to keep
    'formatting consistent between text editors.
    Dim i As Integer
    Dim OutputData As String
    Dim TestChar As String
    For i = 1 To Len(DataToConvert$)
        TestChar = Mid$(DataToConvert$, i, 1)
        If TestChar = TAB_CHAR Then
            OutputData = OutputData + Space$(4)
        Else
            OutputData = OutputData + TestChar
        End If
    Next i
    ConvertTabsToSpaces$ = OutputData

    'return mouse cursor back to normal
    Screen.MousePointer = 0
End Function

Function CreateDirectory% (DirPathBase$, NewDirName$)
    
    If Right$(DirPathBase$, 1) <> "\" Then
        DirPathBase$ = DirPathBase$ + "\"
    End If

    If NewDirName$ = "" Or InStr(NewDirName$, ".") > 0 Then
        MsgBox "Please enter a valid new directory name.", MB_ICONEXCLAMATION, "Invalid Directory Name"
        CreateDirectory% = False
        Exit Function
    End If

    If Len(NewDirName$) > 8 Then
        NewDirName$ = Left$(NewDirName$, 6) + "~1"
    End If
    
    On Error GoTo DirCreateErrorHandler
    MkDir DirPathBase$ + NewDirName$

    CreateDirectory% = True
    Exit Function

DirCreateErrorHandler:   ' Error handler line label.
    Dim Msg As String
    Select Case Err
        Case 0: Exit Function' err 0 raises each time. ignore.
        Case 53: Msg = "ERROR 53: That file doesn't exist."
        Case 68: Msg = "ERROR 68: Drive not available."
        Case 75: Msg = "ERROR 75: Directory already exists."
        Case 76: Msg = "ERROR 76: Invalid Path."
        Case Else: Msg = "ERROR " & Err & " occurred."
    End Select
    MsgBox Msg, MB_ICONEXCLAMATION, "Error Creating Directory"  ' Display error message.
    CreateDirectory% = False
    Exit Function

End Function

Function DeleteDirectory% (Path$)

    If Len(Path$) = 3 Then
        MsgBox "Cannot delete root directory", MB_ICONEXCLAMATION, "Error"
        DeleteDirectory% = False
        Exit Function
    End If

    Dim Confirm As Integer

    Confirm = MsgBox("Are you sure you want to delete: " & NEW_LINE & Path$, MB_ICONQUESTION + MB_YESNO, "Delete Directory")

    If Confirm = IDYES Then
        On Error GoTo DeleteDirectoryErrorHandler
        RmDir Path$
        DeleteDirectory% = True
        Exit Function
    End If

    DeleteDirectory% = False
    Exit Function

DeleteDirectoryErrorHandler:   ' Error handler line label.
    Dim Msg As String
    Select Case Err
        Case 0: Exit Function' err 0 raises each time. ignore.
        Case 53: Msg = "ERROR 53: That file doesn't exist."
        Case 68: Msg = "ERROR 68: Drive not available."
        Case 75: Msg = "ERROR 75: Directory not empty."
        Case 76: Msg = "ERROR 76: That path doesn't exist."
        Case Else: Msg = "ERROR " & Err & " occurred." & NEW_LINE & Error$(Err)
    End Select
    MsgBox Msg, MB_ICONEXCLAMATION, "Error Deleting Directory"  ' Display error message.
    DeleteDirectory% = False
    Exit Function


End Function

Function DeleteFile% (Path$, FileName$)

    If FileName$ = "" Then
        MsgBox "Please select a file to delete.", MB_ICONEXCLAMATION, "Error"
        DeleteFile% = False
        Exit Function
    End If

    If Right$(Path$, 1) <> "\" Then
        Path$ = Path$ + "\"
    End If

    Dim FullPath As String
    FullPath = Path$ + FileName$

    Dim Confirm As Integer
    Confirm = MsgBox("Are you sure you want to delete file:" & NEW_LINE & FullPath, MB_ICONQUESTION + MB_YESNO, "Delete File")
    
    If Confirm <> IDYES Then
        DeleteFile% = False
        Exit Function
    End If

    On Error GoTo DeleteFileErrorHandler
    Kill FullPath

    DeleteFile% = True

    If PREVIOUS_FILE_NAME = FileName$ Then
        Call ResetFileValues(TEMPLATE_TYPE_NONE)
    End If

    Exit Function

DeleteFileErrorHandler:
    Dim Msg As String
    Select Case Err
        Case 0: Exit Function' err 0 raises each time. ignore.
        Case 53: Msg = "ERROR 53: That file doesn't exist."
        Case 68: Msg = "ERROR 68: Drive not available."
        Case 75: Msg = "ERROR 75: Directory not empty."
        Case 76: Msg = "ERROR 76: That path doesn't exist."
        Case Else: Msg = "ERROR " & Err & " occurred."
    End Select
    MsgBox Msg, MB_ICONEXCLAMATION, "Error Deleting File"  ' Display error message.
    DeleteFile% = False
    Exit Function


End Function

Function GetFileContents$ (Path$, FileName$)

    On Error GoTo GetFileContents_ErrorHandler

    'Hour glass mouse cursor during loading
    Screen.MousePointer = 11

    Dim FullPath As String
    Dim FileContents As String
    Dim LineData As String

    FullPath = Path$ + FileName$

    LoadingForm.Visible = True
    FileContents = ""

    Open FullPath For Input Lock Write As #1
        Do While Not EOF(1)
            Line Input #1, LineData
            FileContents = FileContents & LineData & NEW_LINE
        Loop
    Close #1

    Unload LoadingForm
    
    FILE_CONTENTS = FileContents
    PREVIOUS_FULL_PATH = FullPath 'update to new path
    PREVIOUS_PATH = Path$
    PREVIOUS_FILE_NAME = FileName$
    GetFileContents$ = FileContents

    'cursor back to normal when done
    Screen.MousePointer = 0

GetFileContents_ErrorHandler:
    If Err <> 0 Then
        MsgBox "Unrecoverable error loading file contents. File may be too large for editor to edit." & NEW_LINE & NEW_LINE & "ERROR " & Str(Err) & ": " & Error$(Err), MB_ICONSTOP, "Error Loading File"
        Call ResetFileValues(TEMPLATE_TYPE_NONE)
        Screen.MousePointer = 0
        End
    End If

End Function

Function GetFileTemplate$ (TemplateType%)
    'Get template from template data file

    If TemplateType% = TEMPLATE_TYPE_NONE Then
        GetFileTemplate$ = ""
        Exit Function
    End If

    On Error GoTo GetFileTemplateErrorHandler

    Dim Template As String
    Dim FileName As String

    Select Case TemplateType%
        Case TEMPLATE_TYPE_SOURCE
            FileName = TEMPLATE_SOURCE_FILE_NAME
        Case TEMPLATE_TYPE_BUILD
            FileName = TEMPLATE_BUILD_FILE_NAME
        Case Else
            MsgBox "Invalid template type. Cannot load type: " & TemplateType%, MB_ICONEXCLAMATION, "Error loading template"
            GetFileTemplate$ = ""
            Exit Function
    End Select
    
    Dim LineInput As String
    Open FileName For Input As #1
        Do While Not EOF(1)
            Line Input #1, LineInput
            Template = Template & LineInput & NEW_LINE
        Loop
    Close #1
    
    GetFileTemplate$ = Template


GetFileTemplateErrorHandler:
    If Err <> 0 Then
        MsgBox "Error loading template data file: " & FileName & NEW_LINE & Error(Err), MB_ICONEXCLAMATION, "Error Loading Template"
        GetFileTemplate$ = ""
        Exit Function
    End If
End Function

Function GetPreviousFileName$ ()
    If VarType(PREVIOUS_FILE_NAME) <> 8 Then
        GetPreviousFileName$ = NEW_FILE_STR
    Else
        GetPreviousFileName$ = PREVIOUS_FILE_NAME
    End If
End Function

Function IsDiffFile% (NewPath$, NewFileName$)
    
    Dim NewFullPath As String
    NewFullPath = NewPath$ + NewFileName$

    If NewFullPath <> PREVIOUS_FULL_PATH Then
        IsDiffFile% = True
    Else
        IsDiffFile% = False
    End If

End Function

Function IsSaveReq% (CurrentContents$)
    If CurrentContents$ <> FILE_CONTENTS Then
        IsSaveReq% = True
    Else
        IsSaveReq% = False
    End If
    
End Function

Sub ResetFileValues (TemplateType%)
    
    FILE_CONTENTS = GetFileTemplate$(TemplateType%)
    
    PREVIOUS_FILE_NAME = NEW_FILE_STR
    PREVIOUS_PATH = ""
    PREVIOUS_FULL_PATH = ""

    MainForm.Caption = "NE COBOL Editor - " & NEW_FILE_STR
    MainForm.EditorText.Text = FILE_CONTENTS
End Sub

Function SaveFile% (Path$, FileName$, FileContents$)

    Dim FullPath As String

    'make sure always dealing with same case
    Path$ = LCase$(Path$)
    FileName$ = LCase$(FileName$)
    FullPath = Path$ + FileName$

    If IsTabConvertEnabled%() Then
        FileContents$ = ConvertTabsToSpaces$(FileContents$)
    End If

    'hour glass cursor while saving.
    Screen.MousePointer = 11
    Open FullPath For Output Lock Write As #1
        Print #1, FileContents$
    Close #1
    'back to normal
    Screen.MousePointer = 0

    MainForm.EditorText.Text = FileContents$
    FILE_CONTENTS = FileContents$
    PREVIOUS_FULL_PATH = FullPath 'update to new path
    PREVIOUS_PATH = Path$
    PREVIOUS_FILE_NAME = FileName$

    SaveFile% = True

End Function

Sub SavePreviousFile (UpdatedContents$)
    
    FILE_CONTENTS = UpdatedContents$
    
    Dim FileName As String
    FileName = PREVIOUS_FILE_NAME
    If Not SaveFile%(PREVIOUS_PATH, FileName, FILE_CONTENTS) Then
        MsgBox "Failed to save " & PREVIOUS_FILE_NAME & ". Please try again."
    End If
End Sub

Function SearchText% (Start As Integer, Search As String)

    PREV_SEARCH_TEXT = Search

    If Search = "" Then
        SearchText% = False
        Exit Function
    End If

    Dim TextToSearch As String
    Dim FoundWhere As Integer
    
    TextToSearch = MainForm.EditorText.Text

    FoundWhere = InStr(Start, TextToSearch, Search)
    If FoundWhere Then
        MainForm.EditorText.SelStart = FoundWhere - 1
        MainForm.EditorText.SelLength = Len(Search)
        PREV_SEARCH_IDX = FoundWhere + Len(Search) + 1
        SearchText% = True
    Else
        SearchText% = False
    End If

End Function

